@@ -1,4 +1,4 @@
-/* $Id: sha1.c 1067 2011-03-15 18:57:16Z wojtekka $ */
+/* $Id$ */
 
 /*
  *  (C) Copyright 2007 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -26,11 +26,13 @@
  * \brief Funkcje wyznaczania skrótu SHA1
  */
 
+#include <errno.h>
 #include <string.h>
-#include <sys/types.h>
-#include <unistd.h>
 
 #include "libgadu.h"
+#include "internal.h"
+#include "fileio.h"
+#include "config.h"
 
 /** \cond ignore */
 
@@ -38,6 +40,16 @@
 
 #include <openssl/sha.h>
 
+#elif defined(GG_CONFIG_HAVE_GNUTLS)
+
+#include <gnutls/gnutls.h>
+#include <gnutls/crypto.h>
+
+#define SHA_CTX gnutls_hash_hd_t
+#define SHA1_Init(ctx) (gnutls_hash_init((ctx), GNUTLS_DIG_SHA1) == 0 ? 1 : 0)
+#define SHA1_Update(ctx, ptr, len) (gnutls_hash(*(ctx), (ptr), (len)) == 0 ? 1 : 0)
+#define SHA1_Final(digest, ctx) (gnutls_hash_deinit(*(ctx), (digest)), 1)
+
 #else
 
 /*
@@ -60,8 +72,6 @@ A million repetitions of "a"
 /* #define LITTLE_ENDIAN * This should be #define'd if true. */
 /* #define SHA1HANDSOFF * Copies data before messing with it. */
 
-#include <string.h>
-
 typedef struct {
     uint32_t state[5];
     uint32_t count[2];
@@ -69,29 +79,31 @@ typedef struct {
 } SHA_CTX;
 
 static void SHA1_Transform(uint32_t state[5], const unsigned char buffer[64]);
-static void SHA1_Init(SHA_CTX* context);
-static void SHA1_Update(SHA_CTX* context, const unsigned char* data, unsigned int len);
-static void SHA1_Final(unsigned char digest[20], SHA_CTX* context);
+static int SHA1_Init(SHA_CTX* context);
+static int SHA1_Update(SHA_CTX* context, const unsigned char* data, unsigned int len);
+static int SHA1_Final(unsigned char digest[20], SHA_CTX* context);
 
 #define rol(value, bits) (((value) << (bits)) | ((value) >> (32 - (bits))))
 
 /* blk0() and blk() perform the initial expand. */
 /* I got the idea of expanding during the round function from SSLeay */
 #ifndef GG_CONFIG_BIGENDIAN
-#define blk0(i) (block->l[i] = (rol(block->l[i],24)&0xFF00FF00) \
-    |(rol(block->l[i],8)&0x00FF00FF))
+#define blk0(i) (block.l[i] = (rol(block.l[i], 24) & 0xFF00FF00) \
+    |(rol(block.l[i], 8) & 0x00FF00FF))
 #else
-#define blk0(i) block->l[i]
+#define blk0(i) block.l[i]
 #endif
-#define blk(i) (block->l[i&15] = rol(block->l[(i+13)&15]^block->l[(i+8)&15] \
-    ^block->l[(i+2)&15]^block->l[i&15],1))
+#define blk(i) (block.l[i&15] = rol(block.l[(i+13)&15]^block.l[(i+8)&15] \
+    ^block.l[(i+2)&15]^block.l[i&15], 1))
 
 /* (R0+R1), R2, R3, R4 are the different operations used in SHA1 */
+/* style:comma:start-ignore */
 #define R0(v,w,x,y,z,i) z+=((w&(x^y))^y)+blk0(i)+0x5A827999+rol(v,5);w=rol(w,30);
 #define R1(v,w,x,y,z,i) z+=((w&(x^y))^y)+blk(i)+0x5A827999+rol(v,5);w=rol(w,30);
 #define R2(v,w,x,y,z,i) z+=(w^x^y)+blk(i)+0x6ED9EBA1+rol(v,5);w=rol(w,30);
 #define R3(v,w,x,y,z,i) z+=(((w|x)&y)|(w&x))+blk(i)+0x8F1BBCDC+rol(v,5);w=rol(w,30);
 #define R4(v,w,x,y,z,i) z+=(w^x^y)+blk(i)+0xCA62C1D6+rol(v,5);w=rol(w,30);
+/* style:comma:end-ignore */
 
 
 /* Hash a single 512-bit block. This is the core of the algorithm. */
@@ -103,10 +115,8 @@ typedef union {
     unsigned char c[64];
     uint32_t l[16];
 } CHAR64LONG16;
-CHAR64LONG16* block;
-static unsigned char workspace[64];
-    block = (CHAR64LONG16*)workspace;
-    memcpy(block, buffer, 64);
+    CHAR64LONG16 block;
+    memcpy(&block, buffer, sizeof(block));
     /* Copy context->state[] to working vars */
     a = state[0];
     b = state[1];
@@ -114,6 +124,7 @@ static unsigned char workspace[64];
     d = state[3];
     e = state[4];
     /* 4 rounds of 20 operations each. Loop unrolled. */
+    /* style:comma:start-ignore */
     R0(a,b,c,d,e, 0); R0(e,a,b,c,d, 1); R0(d,e,a,b,c, 2); R0(c,d,e,a,b, 3);
     R0(b,c,d,e,a, 4); R0(a,b,c,d,e, 5); R0(e,a,b,c,d, 6); R0(d,e,a,b,c, 7);
     R0(c,d,e,a,b, 8); R0(b,c,d,e,a, 9); R0(a,b,c,d,e,10); R0(e,a,b,c,d,11);
@@ -134,6 +145,7 @@ static unsigned char workspace[64];
     R4(c,d,e,a,b,68); R4(b,c,d,e,a,69); R4(a,b,c,d,e,70); R4(e,a,b,c,d,71);
     R4(d,e,a,b,c,72); R4(c,d,e,a,b,73); R4(b,c,d,e,a,74); R4(a,b,c,d,e,75);
     R4(e,a,b,c,d,76); R4(d,e,a,b,c,77); R4(c,d,e,a,b,78); R4(b,c,d,e,a,79);
+    /* style:comma:end-ignore */
     /* Add the working vars back into context.state[] */
     state[0] += a;
     state[1] += b;
@@ -147,7 +159,7 @@ static unsigned char workspace[64];
 
 /* SHA1_Init - Initialize new context */
 
-static void SHA1_Init(SHA_CTX* context)
+static int SHA1_Init(SHA_CTX* context)
 {
     /* SHA1 initialization constants */
     context->state[0] = 0x67452301;
@@ -156,12 +168,14 @@ static void SHA1_Init(SHA_CTX* context)
     context->state[3] = 0x10325476;
     context->state[4] = 0xC3D2E1F0;
     context->count[0] = context->count[1] = 0;
+
+    return 1;
 }
 
 
 /* Run your data through this. */
 
-static void SHA1_Update(SHA_CTX* context, const unsigned char* data, unsigned int len)
+static int SHA1_Update(SHA_CTX* context, const unsigned char* data, unsigned int len)
 {
 unsigned int i, j;
 
@@ -178,12 +192,14 @@ unsigned int i, j;
     }
     else i = 0;
     memcpy(&context->buffer[j], &data[i], len - i);
+
+    return 1;
 }
 
 
 /* Add padding and return the message digest. */
 
-static void SHA1_Final(unsigned char digest[20], SHA_CTX* context)
+static int SHA1_Final(unsigned char digest[20], SHA_CTX* context)
 {
 uint32_t i, j;
 unsigned char finalcount[8];
@@ -192,9 +208,9 @@ unsigned char finalcount[8];
         finalcount[i] = (unsigned char)((context->count[(i >= 4 ? 0 : 1)]
          >> ((3-(i & 3)) * 8) ) & 255);  /* Endian independent */
     }
-    SHA1_Update(context, (unsigned char *)"\200", 1);
+    SHA1_Update(context, (const unsigned char *)"\200", 1);
     while ((context->count[0] & 504) != 448) {
-        SHA1_Update(context, (unsigned char *)"\0", 1);
+        SHA1_Update(context, (const unsigned char *)"\0", 1);
     }
     SHA1_Update(context, finalcount, 8);  /* Should cause a SHA1_Transform() */
     for (i = 0; i < 20; i++) {
@@ -210,6 +226,8 @@ unsigned char finalcount[8];
 #ifdef SHA1HANDSOFF  /* make SHA1_Transform overwrite it's own static vars */
     SHA1_Transform(context->state, context->buffer);
 #endif
+
+    return 1;
 }
 
 #endif /* GG_CONFIG_HAVE_OPENSSL */
@@ -219,38 +237,102 @@ unsigned char finalcount[8];
 /** \cond internal */
 
 /**
- * \internal Liczy skrĂłt SHA1 z ziarna i hasĹa.
+ * \internal Liczy skrót SHA1 z ziarna i hasła.
  *
- * \param password HasĹo
+ * \param password Hasło
  * \param seed Ziarno
- * \param result Bufor na wynik funkcji skrĂłtu (20 bajtĂłw)
+ * \param result Bufor na wynik funkcji skrótu (20 bajtów)
+ *
+ * \return 0 lub -1
  */
-void gg_login_hash_sha1(const char *password, uint32_t seed, uint8_t *result)
+int gg_login_hash_sha1_2(const char *password, uint32_t seed, uint8_t *result)
 {
 	SHA_CTX ctx;
-	
-	SHA1_Init(&ctx);
-	SHA1_Update(&ctx, (const unsigned char*) password, strlen(password));
+
+	if (!SHA1_Init(&ctx))
+		return -1;
+
+	if (!SHA1_Update(&ctx, (const unsigned char*) password, strlen(password)))
+		goto fail;
+
 	seed = gg_fix32(seed);
-	SHA1_Update(&ctx, (uint8_t*) &seed, 4);
-	
-	SHA1_Final(result, &ctx);
+	if (!SHA1_Update(&ctx, (uint8_t*) &seed, 4))
+		goto fail;
+
+	if (!SHA1_Final(result, &ctx))
+		return -1;
+
+	return 0;
+
+fail:
+	/* Zwolnij zasoby. Tylko GnuTLS przyjęłoby NULL zamiast result, więc przekaż result. */
+	(void)SHA1_Final(result, &ctx);
+	return -1;
 }
 
 /**
- * \internal Liczy skrĂłt SHA1 z pliku.
+ * \internal Liczy skrót SHA1 z fragmentu pliku.
  *
  * \param fd Deskryptor pliku
- * \param result WskaĹşnik na skrĂłt
+ * \param ctx Kontekst SHA-1
+ * \param pos Położenie fragmentu pliku
+ * \param len Długość fragmentu pliku
+ *
+ * \return 0 lub -1
+ */
+static int gg_file_hash_sha1_part(int fd, SHA_CTX *ctx, off_t pos, size_t len)
+{
+	unsigned char buf[4096];
+	size_t chunk_len;
+	int res = 0;
+
+	while (len > 0) {
+		if (lseek(fd, pos, SEEK_SET) == (off_t) -1) {
+			res = -1;
+			break;
+		}
+
+		chunk_len = len;
+
+		if (chunk_len > sizeof(buf))
+			chunk_len = sizeof(buf);
+
+		res = read(fd, buf, chunk_len);
+
+		if (res == -1 && errno != EINTR)
+			break;
+
+		if (res != -1) {
+			if (!SHA1_Update(ctx, buf, res)) {
+				res = -1;
+				break;
+			}
+
+			pos += res;
+			len -= res;
+		}
+	}
+
+	return res;
+}
+
+/**
+ * \internal Liczy skrót SHA1 z pliku.
+ *
+ * Dla plików poniżej 10MB liczony jest skrót z całego pliku, dla plików
+ * powyżej 10MB liczy się 9 jednomegabajtowych fragmentów.
+ *
+ * \param fd Deskryptor pliku
+ * \param result Bufor na wynik funkcji skrótu (20 bajtów)
  *
  * \return 0 lub -1
  */
 int gg_file_hash_sha1(int fd, uint8_t *result)
 {
-	unsigned char buf[4096];
 	SHA_CTX ctx;
 	off_t pos, len;
 	int res;
+	const size_t part_len = 1048576;
 
 	if ((pos = lseek(fd, 0, SEEK_CUR)) == (off_t) -1)
 		return -1;
@@ -261,38 +343,29 @@ int gg_file_hash_sha1(int fd, uint8_t *result)
 	if (lseek(fd, 0, SEEK_SET) == (off_t) -1)
 		return -1;
 
-	SHA1_Init(&ctx);
+	if (!SHA1_Init(&ctx))
+		return -1;
 
-	if (len <= 10485760) {
-		while ((res = read(fd, buf, sizeof(buf))) > 0)
-			SHA1_Update(&ctx, buf, res);
+	if (len <= (off_t)part_len * 10) {
+		res = gg_file_hash_sha1_part(fd, &ctx, 0, len);
 	} else {
-		int i;
+		unsigned int i;
 
 		for (i = 0; i < 9; i++) {
-			int j;
+			off_t part_pos = (len - part_len) / 9 * i;
 
-			if (lseek(fd, (len - 1048576) / 9 * i, SEEK_SET) == (off_t) - 1)
-				return -1;
-
-			for (j = 0; j < 1048576 / sizeof(buf); j++) {
-				if ((res = read(fd, buf, sizeof(buf))) != sizeof(buf)) {
-					res = -1;
-					break;
-				}
-
-				SHA1_Update(&ctx, buf, res);
-			}
+			res = gg_file_hash_sha1_part(fd, &ctx, part_pos, part_len);
 
 			if (res == -1)
 				break;
 		}
 	}
 
-	if (res == -1)
+	if (!SHA1_Final(result, &ctx))
 		return -1;
 
-	SHA1_Final(result, &ctx);
+	if (res == -1)
+		return -1;
 
 	if (lseek(fd, pos, SEEK_SET) == (off_t) -1)
 		return -1;
