@@ -619,14 +619,16 @@ static void msim_username_is_set_cb(MsimSession *session, const MsimMessage *use
 	MsimMessage *body;
 
 	guint rid;
-	gint cmd,dsn,uid,lid,code;
+	gint cmd,dsn,lid,code;
 	/* \persistr\\cmd\258\dsn\9\uid\204084363\lid\14\rid\369\body\UserName=TheAlbinoRhino1.Code=0\final\ */
 
 	purple_debug_info("msim","username_is_set made\n");
 
 	cmd = msim_msg_get_integer(userinfo, "cmd");
 	dsn = msim_msg_get_integer(userinfo, "dsn");
+#if 0
 	uid = msim_msg_get_integer(userinfo, "uid");
+#endif
 	lid = msim_msg_get_integer(userinfo, "lid");
 	body = msim_msg_get_dictionary(userinfo, "body");
 	errmsg = _("An error occurred while trying to set the username.  "
