@@ -460,7 +460,8 @@ static void yahoo_codes_to_html_add_tag(xmlnode **cur, const char *tag, gboolean
 		GData *attributes;
 		char *fontsize = NULL;
 
-		purple_markup_find_tag(tag_name, tag, &start, &end, &attributes);
+		if (!purple_markup_find_tag(tag_name, tag, &start, &end, &attributes))
+			g_return_if_reached();
 		*cur = xmlnode_new_child(*cur, tag_name);
 
 		if (is_font_tag) {
@@ -738,7 +739,8 @@ static void parse_font_tag(GString *dest, const char *tag_name, const char *tag,
 	gboolean needendtag;
 	GString *tmp;
 
-	purple_markup_find_tag(tag_name, tag, &start, &end, &attributes);
+	if (!purple_markup_find_tag(tag_name, tag, &start, &end, &attributes))
+		g_return_if_reached();
 
 	needendtag = FALSE;
 	tmp = g_string_new(NULL);
@@ -864,7 +866,16 @@ char *yahoo_html_to_codes(const char *src)
 					 */
 
 					/* Append the URL */
-					purple_markup_find_tag(tag_name, tag, &start, &end, &attributes);
+					if (!purple_markup_find_tag(tag_name,
+						tag, &start, &end, &attributes))
+					{
+						g_warn_if_reached();
+						i = j;
+						g_free(tag);
+						g_free(tag_name);
+						break;
+					}
+
 					attribute = g_datalist_get_data(&attributes, "href");
 					if (attribute != NULL) {
 						if (purple_str_has_prefix(attribute, "mailto:"))
