@@ -136,8 +136,13 @@ static gboolean timeoutfunc(gpointer data) {
 	}
 	purple_debug_info("stun", "request timed out, retrying.\n");
 	sc->retry++;
-	sendto(sc->fd, sc->packet, sc->packetsize, 0,
-		(struct sockaddr *)&(sc->addr), sizeof(struct sockaddr_in));
+	if (sendto(sc->fd, sc->packet, sc->packetsize, 0,
+		(struct sockaddr *)&(sc->addr), sizeof(struct sockaddr_in)) !=
+		sc->packetsize)
+	{
+		purple_debug_warning("stun", "sendto failed\n");
+		return FALSE;
+	}
 	return TRUE;
 }
 
@@ -174,6 +179,8 @@ static void reply_cb(gpointer data, gint source, PurpleInputCondition cond) {
 	struct ifreq *ifr;
 	struct sockaddr_in *sinptr;
 
+	memset(&in, 0, sizeof(in));
+
 	len = recv(source, buffer, sizeof(buffer) - 1, 0);
 	if (len < 0) {
 		purple_debug_warning("stun", "unable to read stun response\n");
