@@ -319,9 +319,9 @@ icon_box_press_cb(GtkWidget *widget, GdkEventButton *event, PidginStatusBox *box
 
 		box->icon_box_menu = gtk_menu_new();
 
-		menu_item = pidgin_new_item_from_stock(box->icon_box_menu, _("Select Buddy Icon"), GTK_STOCK_ADD,
-						     G_CALLBACK(choose_buddy_icon_cb),
-						     box, 0, 0, NULL);
+		pidgin_new_item_from_stock(box->icon_box_menu,
+			_("Select Buddy Icon"), GTK_STOCK_ADD,
+			G_CALLBACK(choose_buddy_icon_cb), box, 0, 0, NULL);
 
 		menu_item = pidgin_new_item_from_stock(box->icon_box_menu, _("Remove"), GTK_STOCK_REMOVE,
 						     G_CALLBACK(remove_buddy_icon_cb),
@@ -2724,12 +2724,16 @@ get_statusbox_index(PidginStatusBox *box, PurpleSavedStatus *saved_status)
 		/* In reverse order */
 		case PURPLE_STATUS_OFFLINE:
 			index++;
+			/* fall through */
 		case PURPLE_STATUS_INVISIBLE:
 			index++;
+			/* fall through */
 		case PURPLE_STATUS_UNAVAILABLE:
 			index++;
+			/* fall through */
 		case PURPLE_STATUS_AWAY:
 			index++;
+			/* fall through */
 		case PURPLE_STATUS_AVAILABLE:
 			index++;
 			break;
