@@ -131,7 +131,10 @@ get_element_devices(const gchar *element_name)
 			const gchar *name;
 			const gchar *device_name;
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+			/* GValueArray is in gstreamer-0.10 API */
 			device = g_value_array_get_nth(array, n);
+G_GNUC_END_IGNORE_DEPRECATIONS
 			g_object_set_property(G_OBJECT(element), "device", device);
 			if (gst_element_set_state(element, GST_STATE_READY)
 					!= GST_STATE_CHANGE_SUCCESS) {
